
/**
 * Copyright Manning Publications Co.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific lan      
*/

import org.apache.lucene.analysis.standard.ClassicAnalyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.TextField;
import org.apache.lucene.document.StringField;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.queryparser.classic.MultiFieldQueryParser;
import org.apache.lucene.queryparser.classic.QueryParser;
import org.apache.lucene.search.Explanation;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.similarities.Similarity;
import org.apache.lucene.search.similarities.DefaultSimilarity;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.Directory;
import org.apache.lucene.util.Version;

import java.io.File;
import java.util.HashMap;

// From chapter 3
public class Explainer {
 
  public static String repeatStr(String str, int n) {
	  String result = "";
	  for(int i = 1; i < n; i++) {
		  result = result + str + " ";
	  }
	  return result + str;
  }
  
  public static void main(String[] args) throws Exception {
    if (args.length != 2) {
      System.err.println("Usage: Explainer <index dir> <query>");
      System.exit(1);
    }

    String indexDir = args[0];
    String queryExpression = args[1];

    Directory dir = FSDirectory.open(new File(indexDir));
    DirectoryReader reader = DirectoryReader.open(dir);
    
    reader = DirectoryReader.open(dir);
    System.out.println(reader.numDocs());
   
    String[] fields = new String[] {"contents", "filename"};
    HashMap<String, Float> boosts = new HashMap<String, Float>();
    boosts.put("contents", 1.1f);
    boosts.put("filename", 1.2f);
    MultiFieldQueryParser multiparser = new MultiFieldQueryParser(Version.LUCENE_45, fields, 
    	    new StandardAnalyzer(Version.LUCENE_45), boosts);

    Query query = multiparser.parse(queryExpression);
    System.out.println("Query: " + queryExpression);    
    
    IndexSearcher searcher = new IndexSearcher(reader);

    TopDocs topDocs = searcher.search(query, 10);

    for (ScoreDoc match : topDocs.scoreDocs) {
      Explanation explanation
         = searcher.explain(query, match.doc);     //#A

      System.out.println("----------");
      Document doc = searcher.doc(match.doc);
      System.out.println(doc.get("filename"));
      System.out.println(explanation.toString());  //#B
    }
    reader.close();
    dir.close();
  }
}
/*
#A Generate Explanation
#B Output Explanation
*/
